///////////////////////////////////////////////////////////////////////////
// Workfile : ATMega16_32.c
// Author : Georg Gschwandtner, Hubert Gschwandtner
// Date : 17. 07. 2011
// Description : Testprogramm fr ATMEL Evaluations- Addonboard
// Remarks : UART- und LCD-Library von P.Fleury
//			Tasten-HW gendert, alle Tasten schalten nach GND
// Revision : 0
///////////////////////////////////////////////////////////////////////////

#include <avr/io.h> 
#include <util/delay.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "lcd_new.h"

#define UART_BAUD_RATE     2400 // Bautrate UART    

void ledeinaus(volatile unsigned char *reg, unsigned char port){
	*reg |= (1<<port);
	_delay_ms(200);
	*reg &= ~(1<<port);
	_delay_ms(200);
}

int main(void){
	DDRD= (1<<DDD3)|(1<<DDD4)|(1<<DDD5)|(1<<DDD6)|(1<<DDD7);			// Ausgnge aktiv
	DDRA=(1<<DDA0)| (1<<DDA2)|(1<<DDA3)|(1<<DDA4)|(1<<DDA5);			// Ausgnge aktiv
	PORTA= (1<<PA1); // PullUp aktiv
	uart_init(UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU));  //UART parametrieren
	lcd_init(LCD_DISP_ON);
	sei(); // Interruptaktivierung

//Test start
	lcd_puts("Test start");
	uart_puts("Test start\n\r");	
	_delay_ms(500);
	lcd_clrscr();
	lcd_puts("Summer Test"); 
	uart_puts("Summer Test\n\r");
	for ( int i = 0; i<500;i++){ 
		PORTD |= (1<<PD7); 
		_delay_ms(1);
		PORTD &=~ (1<<PD7);
		_delay_ms(1);
	}
	_delay_ms(500);
	lcd_clrscr();
		
	lcd_puts("Audio Test"); 
	uart_puts("Audio Test\n\r");
	for ( int i = 0; i<500;i++){ 
		PORTA |= (1<<PA0); 
		_delay_ms(1);
		PORTA &=~ (1<<PA0);
		_delay_ms(1);
	}
	_delay_ms(500);
	lcd_clrscr();

	lcd_puts("LED Test"); 
	uart_puts("LED Test");
	ledeinaus(&PORTA,PA2); 
	ledeinaus(&PORTA,PA3);
	ledeinaus(&PORTA,PA4);
	ledeinaus(&PORTA,PA5);
	ledeinaus(&PORTD,PD3);
	ledeinaus(&PORTD,PD4);
	ledeinaus(&PORTD,PD5);
	ledeinaus(&PORTD,PD6);
	_delay_ms(500);
	lcd_clrscr();
	
	lcd_puts("Tasten Test");
	uart_puts("Tasten Test\n\r");
	int count = 0;
	char flag = 0;
	lcd_clrscr();
	lcd_puts("Taste 1 druecken");
	uart_puts("Taste 1 druecken\n\r");
	while (count<3){
		if (!(PINA & (1<<PINA1))&& (flag== 0)){
			flag = 1;
			count++;
			lcd_clrscr();
			lcd_puts("Taste ");
			lcd_putc(count+48);
			lcd_puts(" mal");
			lcd_gotoxy(0,1);
			lcd_puts(" gedrueckt ");
			uart_puts("Taste ");
			uart_putc(count+48);
			uart_puts(" mal gedrueckt\n\r");
			_delay_ms(200);
		}
		else {
			flag =0;
		}
	}
	lcd_clrscr();
	lcd_puts("UART-Test");
	lcd_gotoxy(0,1);
	lcd_puts("Warte auf Daten");
	uart_puts("UART-Test Warte auf Daten\n\r");
	_delay_ms(500);
	lcd_clrscr();
	unsigned int c= 0;
	while (c !=10){
 		c = uart_getc();
        if ( c & UART_NO_DATA )
        {
            /* 
             * no data available from UART 
             */
        }
        else
        {
            /*
             * new data available from UART
             * check for Frame or Overrun error
             */
            if ( c & UART_FRAME_ERROR )
            {
                /* Framing Error detected, i.e no stop bit detected */
                uart_puts_P("UART Frame Error: ");
            }
            if ( c & UART_OVERRUN_ERROR )
            {
                /* 
                 * Overrun, a character already present in the UART UDR register was 
                 * not read by the interrupt handler before the next character arrived,
                 * one or more received characters have been dropped
                 */
                uart_puts_P("UART Overrun Error: ");
            }
            if ( c & UART_BUFFER_OVERFLOW )
            {
                /* 
                 * We are not reading the receive buffer fast enough,
                 * one or more received character have been dropped 
                 */
                uart_puts_P("Buffer overflow error: ");
            }
            /* 
             * send received character back
             */
			if ( (c != 10)&& (c!=13)){
            	lcd_putc( (unsigned char)c );
			}
        }
	}
	_delay_ms(500);
	lcd_clrscr();
	lcd_puts("UART-Test Ende");
	uart_puts("UART-Test Ende\n\r");


for(;;){
}
}

